# -------------------------------------------------------------------------
#
# Script name: 01_bioclim_selection
#
# Purpose of script: Select bioclimatic indicators for modelling
# Author(s): Bruno M. Carvalho
# Date Created: 2024-06-03
# Email: bruno.carvalho@bsc.es
#
# Load packages -----------------------------------------------------------

packages <- c("readr", "dplyr", "tidyr", "corrplot", "caret", 
              "xgboost", "pROC")
#install.packages(setdiff(packages, rownames(installed.packages())))
lapply(packages, require, character.only = TRUE)
rm(packages)


# Data input --------------------------------------------------------------

# Harmonised model input data
df <- read_csv("data/leish_model_input_data.csv")

# Run ---------------------------------------------------------------------

#### 1. Check correlations between bioclims ####

# only last period because it will be used in model calibration
cor_data <- df %>% 
  filter(period == "2011-2020") %>% 
  select(contains("BIO"))

# generate correlation matrix
cor_matrix <- cor(cor_data)
cor_matrix[upper.tri(cor_matrix)] <- 0 # upper triangle = 0
diag(cor_matrix) <- 0 # diagonal = 0

corrplot(cor_matrix, method = "circle")

#### 2. Run preliminary model to assess variable contributions ####

# restrict climate data to second period for model calibration
mod_df <- df %>% 
  filter(period == "2011-2020") %>%
  # use only leish and bio columns
  select(NUTS_ID, leish, contains("BIO"))

# random 80% train and 20% test partitions
set.seed(123)
trainIndex <- createDataPartition(as.matrix(select(mod_df, leish)),
                                  p = .8, list = FALSE, times = 1)

df_split1_train <- mod_df[ trainIndex,] %>%
  mutate(split1 = "train")

df_split1_test  <- mod_df[-trainIndex,] %>%
  mutate(split1 = "test")

df_split1 <- bind_rows(df_split1_train, df_split1_test) %>%
  select(NUTS_ID, split1)

# combine train/test sets into dense matrices for xgboost
dtrain_leish <- xgb.DMatrix(data = as.matrix(df_split1_train %>% 
                                               select(-NUTS_ID, -leish, -split1)), 
                            label = as.matrix(df_split1_train %>% 
                                                select(leish)))

dtest_leish <- xgb.DMatrix(data = as.matrix(df_split1_test %>% 
                                              select(-NUTS_ID, -leish, -split1)), 
                           label = as.matrix(df_split1_test %>% 
                                               select(leish)))

# get pa weights
pa <- table(getinfo(dtrain_leish, "label"))
pa_weights <- pa[2]/pa[1]

# train and test model
mod_prelim <- xgb.train(data = dtrain_leish,
                      params = list(objective = "binary:logistic", 
                                    eval.metric = "auc",
                                    scale_pos_weight = pa_weights),
                      watchlist = list(train = dtrain_leish, test = dtest_leish), # use test set for evaluation
                      nthread = 2, # number of CPU threads to use
                      nrounds = 1000, # number of iterations (replicates)
                      early_stopping_rounds = 50) # stop if eval metric does not improve for x rounds

# predict into test set
pred <- predict(mod_prelim, dtest_leish)

# generate ROC curves
roc_mod <- roc(response = getinfo(dtest_leish, "label"),
               predictor = pred,
               levels=c(0, 1),
               smooth = FALSE,
               ci=TRUE)
# auc
mod_prelim$best_score

# auc CI
roc_mod$ci

# plot feature contributions 
xgb.plot.importance(importance_matrix = xgb.importance(model = mod_prelim),
                    measure = "Gain", main = "", cex = 1.2, xlim = c(0,0.4))

# most relevant in prelim runs: BIO15, BIO4, BIO1
# final selection: BIO01 BIO02 BIO03	BIO04	BIO05	BIO06	BIO13	BIO14	BIO15
selected_bios <- c("BIO01", "BIO02", "BIO03", "BIO04", "BIO05", 
                   "BIO06", "BIO13", "BIO14", "BIO15")
unselected_bios <- c("BIO07", "BIO08", "BIO09", "BIO10", "BIO11",
                     "BIO12", "BIO16", "BIO17", "BIO18", "BIO19")


# Data output -------------------------------------------------------------

# bioclimatic indicators correlation plot
png(file = paste0("output/01_bioclims_cor_plot.png"),
    width=500, height=500)
corrplot(cor_matrix, method = "circle", cl.cex=1.2)
dev.off()

# feature contributions plot 
png(file = paste0("output/01_prelim_model_varcontrib.png"),
    width=1000, height=600)
par(mar = c(3, 7, 0.2, 2))
xgb.plot.importance(importance_matrix = xgb.importance(model = mod_prelim),
                    measure = "Gain", main = "", cex = 2, cex.axis=1.5,
                    left_margin = NULL, xlim = c(0,0.4))
dev.off()

#END
