# -------------------------------------------------------------------------
#
# Script name: 02_sand_fly_models
#
# Purpose of script: Modelling suitability for Leishmania vectors
# Author(s): Bruno M. Carvalho
# Date Created: 2024-06-03
# Email: bruno.carvalho@bsc.es
#
# Load packages -----------------------------------------------------------

packages <- c("readr", "dplyr", "tidyr", "corrplot", "caret", "xgboost", "pROC")
#install.packages(setdiff(packages, rownames(installed.packages())))
lapply(packages, require, character.only = TRUE)
rm(packages)

# custom function
source("scripts/functions/BestParamSearch.R")

# Data input --------------------------------------------------------------

# Harmonised model input data
df <- read_csv("data/leish_model_input_data.csv")

# define name of model run (to save results)
mod_run = "vectors_mod5"

# selected bioclimatic indicators (from script 01_bioclim_selection)
selected_bios <- c("BIO01", "BIO02", "BIO03", "BIO04", "BIO05", 
                   "BIO06", "BIO13", "BIO14", "BIO15")
unselected_bios <- c("BIO07", "BIO08", "BIO09", "BIO10", "BIO11", 
                     "BIO12", "BIO16", "BIO17", "BIO18", "BIO19")

# Run ---------------------------------------------------------------------

#### 1. Prepare train and test datasets for modelling ####

mod_df <- df %>% 
  # restrict climate data to second period for model calibration
  filter(period == "2011-2020") %>%  
  # removing unused data
  select(-leish, -period) %>% 
  # removing correlated bioclims
  select(-all_of(unselected_bios))

# simple vector species names
sf_names <- c("Pnegl", "Ptobb", "Paria", "Ppern", "Pperf")

set.seed(123)

dtrain_vectors <- list()
dtest_vectors <- list()
dfsplit_vectors <- list()
for(i in 1:length(sf_names)){
  sp_df <- mod_df %>%
    select(-sf_names[!sf_names %in% sf_names[i]])
  
  trainIndex <- createDataPartition(as.matrix(select(sp_df, sf_names[i])), 
                                    p = .8, list = FALSE, times = 1)
  # train and test dfs
  df_train <- sp_df[ trainIndex,]
  df_test  <- sp_df[-trainIndex,]
  
  # separate Y (vector) and X (features) in each set
  Y_train <- df_train %>% 
    select(sf_names[i])
  
  Y_test <- df_test %>% 
    select(sf_names[i])
  
  X_train <- df_train %>% 
    select(-NUTS_ID, -sf_names[i])
  
  X_test <- df_test %>% 
    select(-NUTS_ID, -sf_names[i])
  
  # combine sets into dense matrices for xgboost
  dtrain_vectors[[i]] <- xgb.DMatrix(data = as.matrix(X_train), 
                                     label = as.matrix(Y_train))
  dtest_vectors[[i]] <- xgb.DMatrix(data = as.matrix(X_test), 
                                    label = as.matrix(Y_test))
 
  split_train <- df_train %>% 
    select(1:2) %>% 
    mutate(split = "train") %>% 
    rename(!!paste0(sf_names[i], "_split") := "split")
  
  split_test <- df_test %>% 
    select(1:2) %>% 
    mutate(split = "test") %>% 
    rename(!!paste0(sf_names[i], "_split") := "split")
  
  dfsplit_vectors[[i]] <- bind_rows(split_train, split_test) %>% 
    arrange(NUTS_ID)
}

modrun_split <- Reduce(left_join, dfsplit_vectors)


#### 2. find best xgboost parameter values for each species ####

# set number of CPU cores to use in the parameter tuning function
# parallel::detectCores()
nthread = 4 

# initial parameter values
vals_eta = c(0.001, 0.01, 0.05, 0.1, 0.2, 0.3, 0.4, 0.5)
vals_max_depth = seq(15, 29, by = 2)
vals_max_leaves = c(63, 127, 255, 511, 1023, 2047, 4095)
vals_subsample = seq(0.5, 1, by = 0.1)
vals_colsample_bytree = seq(0.5, 1, by = 0.1)
vals_gamma = seq(0, 20, by = 1)

# find best parameters by 10-fold cross-validation
best_eta <- list()
best_max_depth <- list()
best_max_leaves <- list()
best_subsample <- list()
best_colsample_bytree <- list()
best_gamma <- list()
for(i in 1:length(sf_names)){
  # get pa weights
  pa <- table(getinfo(dtrain_vectors[[i]], "label"))
  pa_weights <- pa[2]/pa[1]
  
  # eta
  param_df <- data.frame(eta = vals_eta)
  search <- BestParamSearch(param_df, dtrain_vectors[[i]], pa_weights, nthread)
  best_eta[[i]] <- search[which.max(search$test_auc_mean),"eta"]
  
  # max depth and leaves
  param_df <- expand.grid(max_depth = vals_max_depth,
                          max_leaves = vals_max_leaves,
                          eta = best_eta[[i]])
  search <- BestParamSearch(param_df, dtrain_vectors[[i]], pa_weights, nthread)
  best_max_depth[[i]] <- search[which.max(search$test_auc_mean),"max_depth"]
  best_max_leaves[[i]] <- search[which.max(search$test_auc_mean),"max_leaves"]
  
  # random sampling params
  param_df <- expand.grid(
    subsample = vals_subsample,
    colsample_bytree = vals_colsample_bytree,
    max_depth = best_max_depth[[i]], 
    max_leaves = best_max_leaves[[i]], 
    eta = best_eta[[i]])
  search <- BestParamSearch(param_df, dtrain_vectors[[i]], pa_weights, nthread)
  best_subsample[[i]] <- search[which.max(search$test_auc_mean),"subsample"]
  best_colsample_bytree[[i]] <- search[which.max(search$test_auc_mean),"colsample_bytree"]
  
  # gamma (controls overfitting)
  param_df <- expand.grid(
    gamma = vals_gamma,
    subsample = best_subsample[[i]],
    colsample_bytree = best_colsample_bytree[[i]],
    max_depth = best_max_depth[[i]], 
    max_leaves = best_max_leaves[[i]], 
    eta = best_eta[[i]])
  search <- BestParamSearch(param_df, dtrain_vectors[[i]], pa_weights, nthread)
  best_gamma[[i]] <- search[which.max(search$test_auc_mean),"gamma"]
}

best_params <- data.frame(species = sf_names,
                          eta = unlist(best_eta),
                          max_depth = unlist(best_max_depth),
                          max_leaves = unlist(best_max_leaves),
                          subsample = unlist(best_subsample),
                          colsample_bytree = unlist(best_colsample_bytree),
                          gamma = unlist(best_gamma))

# OPTIONAL: reload best parameters file
# best_params <- read_csv("output/02_vectors_mod5_best_params.csv")

#### 3. Run final models with best parameters ####

# run models and save outputs
mod <- list()
results <- data.frame(vector_species = sf_names,
                      model_run = mod_run,
                      test_auc = 0,
                      test_auc_ci = 0,
                      best_thres = 0,
                      best_thres_sens = 0,
                      best_thres_spec = 0)
for(i in 1:length(sf_names)){
  # get pa weights
  pa <- table(getinfo(dtrain_vectors[[i]], "label"))
  pa_weights <- pa[2]/pa[1]
  
  # best params
  params_sf <- list(objective = "binary:logistic", 
                    eval.metric = "auc",
                    scale_pos_weight = pa_weights,
                    eta = best_params[best_params$species == sf_names[i], 
                                      "eta"],
                    max_depth = best_params[best_params$species == sf_names[i], 
                                            "max_depth"],
                    max_leaves = best_params[best_params$species == sf_names[i], 
                                             "max_leaves"],
                    subsample = best_params[best_params$species == sf_names[i], 
                                            "subsample"],
                    colsample_bytree = best_params[best_params$species == sf_names[i], 
                                                   "colsample_bytree"],
                    gamma = best_params[best_params$species == sf_names[i], 
                                        "gamma"])
  
  # train and test model
  mod[[i]] <- xgb.train(data = dtrain_vectors[[i]],
                        params = params_sf,
                        watchlist = list(train = dtrain_vectors[[i]], 
                                         test = dtest_vectors[[i]]), # use test set for evaluation
                        nthread = 2, # number of CPU threads to use
                        nrounds = 1000, # number of iterations (replicates)
                        early_stopping_rounds = 50) # stop if eval metric does not improve for x rounds
  
  # predict into test set
  pred <- predict(mod[[i]], dtest_vectors[[i]])
  
  # generate ROC curves
  roc_mod <- roc(response = getinfo(dtest_vectors[[i]], "label"),
                 predictor = pred,
                 levels=c(0, 1),
                 smooth = FALSE,
                 ci=TRUE)

  # store results into df
  results$test_auc[i] <- mod[[i]]$best_score
  results$test_auc_ci[i] <- paste(roc_mod$ci[1], roc_mod$ci[3])
  results$best_thres[i] <- coords(roc_mod, "best")$threshold
  results$best_thres_sens[i] <- coords(roc_mod, "best")$sensitivity
  results$best_thres_spec[i] <- coords(roc_mod, "best")$specificity
  
  # save results df
  write_csv(results, file = paste0("output/02_", mod_run, "_results.csv"))
  
  # save model object
  if(file.exists("output/model_objects") == FALSE){
    dir.create("output/model_objects")
  } else {
    xgb.save(mod[[i]], fname = paste0("output/model_objects/02_", 
                                      mod_run, "_", sf_names[i], "_model"))
  }

}


#### 4. Predictions into study area in both periods ####

# build df for model prediction
mod_df_pred <- df %>% 
  # removing unused columns
  select(-leish) %>% 
  # removing correlated bioclims
  select(-all_of(unselected_bios))

for(i in 1:length(sf_names)){
  cols <- c(sf_names[i], colnames(dtrain_vectors[[i]])) 
  
  sp_pred20_df <- mod_df_pred %>% 
    filter(period == "2011-2020") %>% 
    select(all_of(cols))
  
  sp_pred10_df <- mod_df_pred %>% 
    filter(period == "2001-2010") %>% 
    select(all_of(cols))
  
  dpred20 <- xgb.DMatrix(data = as.matrix(select(sp_pred20_df, -sf_names[i])))
  dpred10 <- xgb.DMatrix(data = as.matrix(select(sp_pred10_df, -sf_names[i])))
  
  pred20 <- data.frame(predict(mod[[i]], dpred20))
  names(pred20) <- paste0(sf_names[i], "_pred")
  
  pred10 <- data.frame(predict(mod[[i]], dpred10)) 
  names(pred10) <- paste0(sf_names[i], "_pred")
  
  pred_sp <- bind_rows(pred20, pred10)
  
  # final df
  mod_df_pred <- bind_cols(mod_df_pred, pred_sp)
  
}

final_preds <- mod_df_pred %>% 
  select("NUTS_ID", "period", all_of(paste0(sf_names, "_pred")))


# Data output -------------------------------------------------------------

# selected best xgboost parameters for each vector species
write_csv(best_params, file = paste0("output/02_", mod_run, "_best_params.csv"))

# predictions for this model run
write_csv(final_preds, file = paste0("output/02_", mod_run, "_predictions.csv"))

#END
