# -------------------------------------------------------------------------
#
# Script name: 03_leish_model_candidates
#
# Purpose of script: Candidate models for leishmaniasis suitability
# Author(s): Bruno M. Carvalho
# Date Created: 2024-06-03
# Email: bruno.carvalho@bsc.es
#
# Load packages -----------------------------------------------------------

packages <- c("readr", "dplyr", "tidyr", "stringr", "caret", "xgboost", "pROC")
#install.packages(setdiff(packages, rownames(installed.packages())))
lapply(packages, require, character.only = TRUE)
rm(packages)

# custom function
source("scripts/functions/BestParamSearch.R")

# Data input --------------------------------------------------------------

# Harmonised model input data
df <- read_csv("data/leish_model_input_data.csv")

# vector covariates (sand fly suitability predictions)
vectdf <- read_csv("output/02_vectors_mod5_predictions.csv")

# selected bioclimatic indicators (from script 01_bioclim_selection)
selected_bios <- c("BIO01", "BIO02", "BIO03", "BIO04", "BIO05", 
                   "BIO06", "BIO13", "BIO14", "BIO15")
unselected_bios <- c("BIO07", "BIO08", "BIO09", "BIO10", "BIO11", 
                     "BIO12", "BIO16", "BIO17", "BIO18", "BIO19")


# Run ---------------------------------------------------------------------

#### 1. Build data frame for model calibration ####

# vector species names
sf_names <- c("Pnegl", "Ptobb", "Paria", "Ppern", "Pperf")

# vector suitability prediction column names
sf_suit_names <- paste0(sf_names, "_pred")

# restrict data to second period for model calibration
mod_df <- df %>% 
  filter(period == "2011-2020")

# covariate sets for model sensitivity analysis
cov1 = sf_names # vectors pa only
cov2 = sf_suit_names # vector suitability
cov3 = selected_bios # climate only
cov4 = c(sf_names, selected_bios) # vectors pa + climate
cov5 = c(sf_suit_names, selected_bios) # vectors suit + climate

# list of covariate sets
input_covs <- list(cov1, cov2, cov3, cov4, cov5)


#### 2. Prepare train and test datasets for modelling ####

# split1: random 80% train and 20% test partitions
set.seed(123)
trainIndex <- createDataPartition(as.matrix(select(mod_df, leish)),
                                  p = .8, list = FALSE, times = 1)

df_split1_train <- mod_df[ trainIndex,] %>%
  mutate(split1 = "train")

df_split1_test  <- mod_df[-trainIndex,] %>%
  mutate(split1 = "test")

df_split1 <- bind_rows(df_split1_train, df_split1_test) %>%
  select(NUTS_ID, split1)

# full train/test split df for saving
modrun_split <- mod_df %>% 
  select(NUTS_ID) %>% 
  left_join(df_split1, by = "NUTS_ID")

# initial parameter values
vals_eta = c(0.001, 0.01, 0.05, 0.1, 0.2, 0.3, 0.4, 0.5)
vals_max_depth = seq(15, 29, by = 2)
vals_max_leaves = c(63, 127, 255, 511, 1023, 2047, 4095)
vals_subsample = seq(0.5, 1, by = 0.1)
vals_colsample_bytree = seq(0.5, 1, by = 0.1)
vals_gamma = seq(0, 20, by = 1)


#### 3. preliminary model runs, random splits ####

# set number of CPU cores to use in the parameter tuning function
#parallel::detectCores()
nthread = 6

# dfs for storing results
best_params <- data.frame(mod_run = character(),
                          response = character(),
                          eta = numeric(),
                          max_depth = numeric(),
                          max_leaves = numeric(),
                          subsample = numeric(),
                          colsample_bytree = numeric(),
                          gamma = numeric())

results <- tibble(mod_run = character(),
                  test_auc = numeric(),
                  test_auc_ci = character(),
                  best_thres = numeric(),
                  best_thres_sens = numeric(),
                  best_thres_spec = numeric())

# run models for each set of test/train and covariates
for(i in 1:length(input_covs)){
  # define name of model run (to save results)
  mod_run = paste0("prelim_cov", i)
  
  # combine train/test sets into dense matrices for xgboost
  dtrain_leish <- xgb.DMatrix(data = as.matrix(df_split1_train %>% 
                                                 select(all_of(input_covs[[i]]))), 
                              label = as.matrix(df_split1_train %>% 
                                                  select(leish)))
  
  dtest_leish <- xgb.DMatrix(data = as.matrix(df_split1_test %>% 
                                                select(all_of(input_covs[[i]]))), 
                             label = as.matrix(df_split1_test %>% 
                                                 select(leish)))
  
  # get pa weights
  pa <- table(getinfo(dtrain_leish, "label"))
  pa_weights <- pa[2]/pa[1]
  
  # eta
  param_df <- data.frame(eta = vals_eta)
  search <- BestParamSearch(param_df, dtrain_leish, pa_weights, nthread)
  best_eta <- search[which.max(search$test_auc_mean),"eta"]
  
  # max depth and leaves
  param_df <- expand.grid(max_depth = vals_max_depth,
                          max_leaves = vals_max_leaves,
                          eta = best_eta)
  search <- BestParamSearch(param_df, dtrain_leish, pa_weights, nthread)
  best_max_depth <- search[which.max(search$test_auc_mean),"max_depth"]
  best_max_leaves <- search[which.max(search$test_auc_mean),"max_leaves"]
  
  # random sampling params
  param_df <- expand.grid(
    subsample = vals_subsample,
    colsample_bytree = vals_colsample_bytree,
    max_depth = best_max_depth, 
    max_leaves = best_max_leaves, 
    eta = best_eta)
  search <- BestParamSearch(param_df, dtrain_leish, pa_weights, nthread)
  best_subsample <- search[which.max(search$test_auc_mean),"subsample"]
  best_colsample_bytree <- search[which.max(search$test_auc_mean),"colsample_bytree"]
  
  # gamma (controls overfitting)
  param_df <- expand.grid(
    gamma = vals_gamma,
    subsample = best_subsample,
    colsample_bytree = best_colsample_bytree,
    max_depth = best_max_depth, 
    max_leaves = best_max_leaves, 
    eta = best_eta)
  search <- BestParamSearch(param_df, dtrain_leish, pa_weights, nthread)
  best_gamma <- search[which.max(search$test_auc_mean),"gamma"]
  
  best_params_run <- data.frame(mod_run = mod_run,
                                response = c("leish"),
                                eta = best_eta,
                                max_depth = best_max_depth,
                                max_leaves = best_max_leaves,
                                subsample = best_subsample,
                                colsample_bytree = best_colsample_bytree,
                                gamma = best_gamma)
  # save best_params to df
  best_params <- bind_rows(best_params, best_params_run)
  
  # load best params
  params_leish <- list(objective = "binary:logistic", 
                       eval.metric = "auc",
                       scale_pos_weight = pa_weights,
                       eta = best_params_run$eta,
                       max_depth = best_params_run$max_depth,
                       max_leaves = best_params_run$max_leaves,
                       subsample = best_params_run$subsample,
                       colsample_bytree = best_params_run$colsample_bytree,
                       gamma = best_params_run$gamma)
  
  # train and test model
  mod <- xgb.train(data = dtrain_leish,
                   params = params_leish,
                   # use test set for evaluation
                   watchlist = list(train = dtrain_leish, test = dtest_leish), 
                   # number of CPU threads to use
                   nthread = nthread, 
                   # number of iterations (replicates)
                   nrounds = 1000, 
                   # stop if eval metric does not improve for x rounds
                   early_stopping_rounds = 50) 
  
  # predict into test set
  pred <- predict(mod, dtest_leish)
  
  # generate ROC curves
  roc_mod <- roc(response = getinfo(dtest_leish, "label"),
                 predictor = pred,
                 levels=c(0, 1),
                 smooth = FALSE,
                 ci=TRUE)
  
  # store results into df
  results_run <- tibble(mod_run = mod_run,
                        test_auc = mod$best_score,
                        test_auc_ci = paste(roc_mod$ci[1], roc_mod$ci[3]),
                        best_thres = coords(roc_mod, "best")$threshold,
                        best_thres_sens = coords(roc_mod, "best")$sensitivity,
                        best_thres_spec = coords(roc_mod, "best")$specificity)
  
  # save results df for this model run
  results <- bind_rows(results, results_run)
  
  # save model object
  xgb.save(mod, fname = paste0("output/model_objects/03_", mod_run, "_model"))
  
}

#### 4. validation runs by country ####

# list of country codes to test validation
# split1: leaving out data from Portugal, Germany, Bulgaria, Denmark
ctry1 = c("PT", "DE", "BG", "DK")
# split2: leaving out data from Spain, Austria, Hungary, Sweden
ctry2 = c("ES", "AT", "HU", "SE")
# split3: leaving out data from Italy, Switzerland, Romania, Norway
ctry3 = c("IT", "CH", "RO", "NO")
# split4: Greece, France, Slovakia, UK
ctry4 = c("EL", "FR", "SK", "UK")

ctry = list(ctry1, ctry2, ctry3, ctry4)
ctry_names = c("group1", "group2", "group3", "group4")

best_cov5 <- best_params %>% 
  filter(mod_run == "prelim_cov5")

results_ctry <- tibble(mod_run = character(),
                       test_auc = numeric(),
                       test_auc_ci = character(),
                       best_thres = numeric(),
                       best_thres_sens = numeric(),
                       best_thres_spec = numeric())

for(i in 1:length(ctry)){
  
  # train/test split for this country
  
  df_split_train <- mod_df %>%
    mutate(country = str_sub(NUTS_ID, end=2)) %>%
    filter(!country %in% ctry[[i]])
  
  dtrain_leish <- xgb.DMatrix(data = as.matrix(df_split_train %>% 
                                                 select(all_of(cov5))), 
                              label = as.matrix(df_split_train %>% 
                                                  select(leish)))
  
  df_split_test <- mod_df %>%
    mutate(country = str_sub(NUTS_ID, end=2)) %>%
    filter(country %in% ctry[[i]])

  dtest_leish <- xgb.DMatrix(data = as.matrix(df_split_test %>% 
                                                select(all_of(cov5))), 
                             label = as.matrix(df_split_test %>% 
                                                 select(leish)))
  
  # get pa weights
  pa <- table(getinfo(dtrain_leish, "label"))
  pa_weights <- pa[2]/pa[1]
  
  # load best params
  params_leish <- list(objective = "binary:logistic", 
                       eval.metric = "auc",
                       scale_pos_weight = pa_weights,
                       eta = best_cov5$eta,
                       max_depth = best_cov5$max_depth,
                       max_leaves = best_cov5$max_leaves,
                       subsample = best_cov5$subsample,
                       colsample_bytree = best_cov5$colsample_bytree,
                       gamma = best_cov5$gamma)
  
  # train and test model
  mod <- xgb.train(data = dtrain_leish,
                   params = params_leish,
                   # use test set for evaluation
                   watchlist = list(train = dtrain_leish, test = dtest_leish), 
                   # number of CPU threads to use
                   nthread = nthread, 
                   # number of iterations (replicates)
                   nrounds = 1000, 
                   # stop if eval metric does not improve for x rounds
                   early_stopping_rounds = 50) 
  
  # predict into test set
  pred <- predict(mod, dtest_leish)
  
  # generate ROC curves
  roc_mod <- roc(response = getinfo(dtest_leish, "label"),
                 predictor = pred,
                 levels=c(0, 1),
                 smooth = FALSE,
                 ci=TRUE)
  
  # store results into df
  results_run <- tibble(mod_run = paste0("valid_", ctry_names[i]),
                        test_auc = mod$best_score,
                        test_auc_ci = paste(roc_mod$ci[1], roc_mod$ci[3]),
                        best_thres = coords(roc_mod, "best")$threshold,
                        best_thres_sens = coords(roc_mod, "best")$sensitivity,
                        best_thres_spec = coords(roc_mod, "best")$specificity)
  
  # save results df for this model run
  results_ctry <- bind_rows(results_ctry, results_run)
  
  # save model object
  xgb.save(mod, fname = paste0("output/model_objects/03_prelim_cov5_model_valid_", ctry_names[i]))
}

# combine final results
results <- bind_rows(results, results_ctry)

# Data output -------------------------------------------------------------

# selected best xgboost parameters for leish prelim models
write_csv(best_params, file = "output/03_prelim_runs_best_params.csv")

# save results df for these model runs
write_csv(results, file = "output/03_prelim_runs_results.csv")

#END
