# -------------------------------------------------------------------------
#
# Script name: 04_leish_final_predictions
#
# Purpose of script: Predicting climatic suitability for Leishmania infantum
# Author(s): Bruno M. Carvalho
# Date Created: 2024-06-03
# Email: bruno.carvalho@bsc.es
#
# Load packages -----------------------------------------------------------

packages <- c("readr", "dplyr", "tidyr", "xgboost", "pROC")
#install.packages(setdiff(packages, rownames(installed.packages())))
lapply(packages, require, character.only = TRUE)
rm(packages)


# Data input --------------------------------------------------------------

# Harmonised model input data
df <- read_csv("data/leish_model_input_data.csv")

# vector covariates (sand fly suitability predictions)
vectdf <- read_csv("output/02_vectors_mod5_predictions.csv")

# choose model run to generate predictions (from 03_leish_model_candidates)
mod_run_name = "prelim_cov5"

# load preliminary model summary results
prelim_results <- read_csv("output/03_prelim_runs_results.csv")

# selected bioclimatic indicators (from script 01_bioclim_selection)
selected_bios <- c("BIO01", "BIO02", "BIO03", "BIO04", "BIO05", 
                   "BIO06", "BIO13", "BIO14", "BIO15")

# load selected model object
mod <- xgb.load(paste0("output/model_objects/03_", mod_run_name, "_model"))

# vector column names
sf_names <- c("Pnegl", "Ptobb", "Paria", "Ppern", "Pperf")
sf_suit_names <- paste0(sf_names, "_pred")

# selected covariate sets (see 03_leish_model_candidates)
cov5 = c(sf_suit_names, selected_bios) # vectors + climate

# Run ---------------------------------------------------------------------

#### 1. Build data frame for model predictions ####

pred_df <- df %>% 
  left_join(vectdf, by = c("NUTS_ID", "period"))


#### 2. Predict into study area in both periods ####

# xgboost dense matrix for predictions
dpred <- xgb.DMatrix(data = as.matrix(pred_df %>% select(all_of(cov5))))

# best threshold value for this model
thres <- prelim_results %>% 
  filter(mod_run == mod_run_name) %>% 
  pull(best_thres)

predictions <- pred_df %>% 
  select(NUTS_ID, period) %>% 
  mutate(
    # continuous predictions for both periods
    leish_pred = predict(mod, dpred),
    # apply threshold for binary predictions
    leish_pred_bin = ifelse(leish_pred > thres, 1, 0)) %>% 
  pivot_wider(names_from = period,
              values_from = c(leish_pred, leish_pred_bin)) %>%
  # tidy column names
  rename(leish_pred20 = `leish_pred_2011-2020`, 
         leish_pred10 = `leish_pred_2001-2010`,
         leish_pred20_bin = `leish_pred_bin_2011-2020`,
         leish_pred10_bin = `leish_pred_bin_2001-2010`) %>% 
  # suitability change between periods
  mutate(change_leish20_10 = leish_pred20 - leish_pred10,
         changebin_final = case_when(leish_pred20_bin == 1 & leish_pred10_bin == 1 ~ 2,
                                     leish_pred20_bin == 1 & leish_pred10_bin == 0 ~ 1,
                                     leish_pred20_bin == 0 & leish_pred10_bin == 1 ~ -1,
                                     leish_pred20_bin == 0 & leish_pred10_bin == 0 ~ 0))

# Data output -------------------------------------------------------------

# final predictions
write_csv(predictions, file = paste0("output/04_final_leish_predictions.csv"))

#END
