# -------------------------------------------------------------------------
#
# Script name: 05_bayesian_models_human
#
# Purpose of script: Associations between suitability and human leishmaniasis
# Author(s): Bruno M. Carvalho, Martín Lotto Batista
# Date Created: 2024-06-03
# Email: bruno.carvalho@bsc.es
#
# Load packages -----------------------------------------------------------

packages <- c("sf", "readr", "dplyr", "tidyr", "stringr",
              "INLA", "spdep", "tibble")
#install.packages(setdiff(packages, rownames(installed.packages())))
lapply(packages, require, character.only = TRUE)
rm(packages)

# custom function
source("scripts/functions/RunINLA.R")

# Data input --------------------------------------------------------------

# study area shapefile
study_area <- st_read("data/study_area.shp")

# clean human VL incidence data
vl_inc <- read_csv("data/human_VL_incidence.csv")

# leish suitability predictions from final model
suit <- read_csv("output/04_final_leish_predictions.csv")


# Run ---------------------------------------------------------------------

#### 1. Prepare input data ####

# Clean and reformat suitability data for modelling
dt.suit <- suit %>% 
  select(NUTS_ID, leish_pred20, leish_pred10) %>% 
  mutate(across(2:3, ~.x*10)) %>% 
  rowwise() %>% 
  mutate(leish_avg=mean(c(leish_pred20, leish_pred10))) %>% 
  ungroup()

# Harmonised df for modelling
dt.model.out <- vl_inc %>% 
  left_join(dt.suit, by="NUTS_ID") %>% 
  # Assign suitability to different periods and countries
  mutate(leish=case_when(country == "France" ~ leish_pred10,
                         country == "Greece" ~ leish_pred20,
                         country == "Spain" ~ leish_avg,
                         country == "Italy" ~ leish_pred20,
                         TRUE ~ NA)) %>% 
  select(NUTS_ID, country, cases, population, leish)

# split into one df per country
dt.model <- dt.model.out %>% 
  # Create a list where every element is a country specific dataset
  group_split(country) %>% 
  purrr::map(~arrange(.x, NUTS_ID)) %>% 
  purrr::map(~mutate(.x, NUTS_ID2=as.numeric(factor(NUTS_ID))))


#### 2. Generate neighborhood structure ####

# Maps to inla graphs
mp <- study_area %>% 
  filter(grepl("ES|FR|EL|IT", NUTS_ID)) %>% # Filter countries of interest
  dplyr::select(NUTS_ID, CNTR_CODE) %>%
  mutate(country=case_when(CNTR_CODE=="FR" ~ "France",
                           CNTR_CODE=="ES" ~ "Spain",
                           CNTR_CODE=="EL" ~ "Greece",
                           CNTR_CODE=="IT" ~ "Italy", TRUE~NA)) %>%
  group_split(country) %>%
  purrr::map(~arrange(.x, NUTS_ID)) %>%
  purrr::map(~mutate(.x, NUTS_ID2=as.numeric(factor(NUTS_ID))))

# Create and save neighborhood structure
nms <- unlist(purrr::map(mp, ~unique(.x[["CNTR_CODE"]])))
# g.file <- NULL
# for(i in 1:length(mp)){
#   nb.map <- spdep::poly2nb(mp[[i]], queen=T)
#   nb2INLA(paste0("data/map_", nms[i], ".graph"), nb.map)
# }

# Read nb matrix as a list
g.file <- lapply(paste0("data/map_", nms, ".graph"), inla.read.graph)


#### 3. Run human disease models ####

# Random effects
res <- "f(NUTS_ID2, model='bym2', graph=g, scale.model=TRUE)"

# Fit baseline model per country (n=4)
form.res <- reformulate(c(1, res), "cases")

m.res <- NULL
for(i in 1:4){
  g <- g.file[[i]]
  m.res[[i]] <- RunHuman(form.res, dt.model[[i]])
  cat(i, "\r")
}

# Fit models with suitability per country (n=4)
m <- NULL
for(i in 1:4){
  g <- g.file[[i]]
  form <- reformulate(c(1, res, "leish"), "cases")
  m[[i]] <- RunHuman(form, dt.model[[i]]) 
  cat(i, "\r")
}


#### 4. Human model analysis ####

# Goodness of fit statistics for each model
gof <- NULL
for(i in 1:4){
  add <- bind_rows(data.frame(model=paste0("baseline_", nms[i]),
                              waic=m.res[[i]][["waic"]][["waic"]],
                              dic=m.res[[i]][["dic"]][["dic"]],
                              log.score.cpo=-mean(log(m.res[[i]][["cpo"]][["cpo"]]), na.rm=TRUE)),
                   data.frame(model=paste0("suitability_", nms[i]),
                              waic=m[[i]][["waic"]][["waic"]],
                              dic=m[[i]][["dic"]][["dic"]],
                              log.score.cpo=-mean(log(m[[i]][["cpo"]][["cpo"]]), na.rm=TRUE)))
  
  gof <- bind_rows(gof, add)
}

# Extract effect sizes
effects <- NULL
for(i in 1:4){
  effects <- bind_rows(effects,
                       m[[i]][["summary.fixed"]] %>% 
                         mutate(model=paste0("suitability_", nms[i])) %>% 
                         rownames_to_column("coefficient") %>% 
                         mutate(across(2:8, ~exp(.x))) %>% 
                         dplyr::select(9,1,2,4,6) %>% 
                         rename(lower=`0.025quant`,
                                upper=`0.975quant`))
}


# Data output -------------------------------------------------------------

# save effect sizes
write_csv(effects, file = "output/05_human_model_effects.csv")

#END
