# -------------------------------------------------------------------------
#
# Script name: 06_bayesian_models_dogs
#
# Purpose of script: Associations between suitability and canine leish
# Author(s): Bruno M. Carvalho, Martín Lotto Batista
# Date Created: 2024-06-03
# Email: bruno.carvalho@bsc.es
#
# Load packages -----------------------------------------------------------

packages <- c("sf", "readr", "dplyr", "tidyr", "stringr",
              "INLA", "spdep", "tibble")
#install.packages(setdiff(packages, rownames(installed.packages())))
lapply(packages, require, character.only = TRUE)
rm(packages)

# custom function
source("scripts/functions/RunINLA.R")

# Data input --------------------------------------------------------------

# study area shapefile
study_area <- st_read("data/study_area.shp")

# clean canine leish prevalence data
canine <- read_csv("data/canine_leish_prevalence.csv")

# leish suitability predictions from final model
suit <- read_csv("output/04_final_leish_predictions.csv")


# Run ---------------------------------------------------------------------

#### 1. Prepare input data ####

# Clean and reformat suitability data for modelling
dt.suit <- suit %>% 
  select(NUTS_ID, leish_pred20, leish_pred10) %>% 
  mutate(across(2:3, ~.x*10)) # Multiply by 10

# Prepare data for canine models
dt.model.out <- canine %>% 
  left_join(dt.suit, by="NUTS_ID") %>% 
  # Assign suitability to different periods and countries
  mutate(leish=case_when(country == "Spain" ~ leish_pred20,
                         country == "Portugal" & period == 2009 ~ leish_pred10,
                         country == "Portugal" & period == 2021 ~ leish_pred20)) %>% 
  select(NUTS_ID, country, tested_dogs, positive_dogs, prevalence, leish)

# Create a list where every element is a country specific dataset
dt.model <- dt.model.out %>% 
  group_split(country) %>% 
  purrr::map(~arrange(.x, NUTS_ID)) %>% 
  purrr::map(~mutate(.x, NUTS_ID2=as.numeric(factor(NUTS_ID))))


#### 2. Generate neighborhood structure ####

# Country codes list
nms <- c("ES", "PT")

# Create neighborhood structures for each country
mp <- study_area %>% 
  # Filter countries of interest
  filter(CNTR_CODE %in% nms) %>% 
  dplyr::select(NUTS_ID, CNTR_CODE) %>% 
  mutate(country=case_when(CNTR_CODE=="PT" ~ "Portugal",
                           CNTR_CODE=="ES" ~ "Spain",
                           CNTR_CODE=="EL" ~ "Greece",
                           TRUE~NA)) %>% 
  # Create list where every object is a map
  group_split(CNTR_CODE) %>% 
  purrr::map(~arrange(.x, NUTS_ID)) %>%
  # new ID column for the models
  purrr::map(~mutate(.x, NUTS_ID2=as.numeric(factor(NUTS_ID))))

# Create and save neighborhood structure
# g.file <- NULL
# for(i in 1:length(mp)){
#   nb.map <- spdep::poly2nb(mp[[i]], queen=T)
#   nb2INLA(paste0("data/map_", nms[i], ".graph"), nb.map)
# }

# Read nb matrix as a list for each model
g.file <- lapply(paste0("data/map_", nms, ".graph"), inla.read.graph)


#### 3. Run canine prevalence models ####

# Random effects
res <- "f(NUTS_ID2, model='bym2', graph=g, scale.model=TRUE)"

# Fit baseline model per country (n=2)
form.res <- reformulate(c(1, res), "positive_dogs")

m.res <- NULL
for(i in 1:length(nms)){
  g <- g.file[[i]]
  m.res[[i]] <- RunCanine(form.res, dt.model[[i]])
  cat(i, "\r")
}

# Fit models with suitability per country (n=2)
m <- NULL
for(i in 1:length(nms)){
  g <- g.file[[i]]
  form <- reformulate(c(1, res, "leish"), "positive_dogs")
  m[[i]] <- RunCanine(form, dt.model[[i]]) 
  cat(i, "\r")
}

#### 4. Canine model analysis ####

# Goodness of fit statistics for each model
gof <- NULL
for(i in 1:2){
  add <- bind_rows(data.frame(model=paste0("baseline_", nms[i]),
                              waic=m.res[[i]][["waic"]][["waic"]],
                              dic=m.res[[i]][["dic"]][["dic"]],
                              log.score.cpo=-mean(log(m.res[[i]][["cpo"]][["cpo"]]), na.rm=TRUE)),
                   data.frame(model=paste0("suitability_", nms[i]),
                              waic=m[[i]][["waic"]][["waic"]],
                              dic=m[[i]][["dic"]][["dic"]],
                              log.score.cpo=-mean(log(m[[i]][["cpo"]][["cpo"]]), na.rm=TRUE)))
  
  gof <- bind_rows(gof, add)
}


# Extract effect sizes
effects <- NULL
for(i in 1:2){
  effects <- bind_rows(effects,
                       m[[i]][["summary.fixed"]] %>% 
                         mutate(model=paste0("suitability_", nms[i])) %>% 
                         rownames_to_column("coefficient") %>% 
                         mutate(across(2:8, ~exp(.x))) %>% 
                         dplyr::select(9,1,2,4,6) %>% 
                         rename(lower=`0.025quant`,
                                upper=`0.975quant`))
}


# Data output -------------------------------------------------------------

# save effect sizes
write_csv(effects, file = "output/06_canine_model_effects.csv")

#END
