# -------------------------------------------------------------------------
#
# Script name: 07_postprocessing.R
#
# Purpose of script: Model output postprocessing and summarising results
# Author(s): Bruno M. Carvalho, Martín Lotto Batista
# Date Created: 2024-06-03
# Email: bruno.carvalho@bsc.es
#
# Load packages -----------------------------------------------------------

packages <- c("readr", "dplyr", "tidyr", "stringr", "sf", "ggplot2", "cowplot")
#install.packages(setdiff(packages, rownames(installed.packages())))
lapply(packages, require, character.only = TRUE)
rm(packages)


# Data input --------------------------------------------------------------

# leish suitability predictions from final model
suit <- read_csv("output/04_final_leish_predictions.csv")

# country info and leish endemicity data
ctry <- read_csv("data/country_endemicity.csv")

# study area shapefile
study_area <- st_read("data/study_area.shp")

# AROPE rates and classification
arope <- read_csv("data/arope_nuts2.csv")

# human VL incidence data
vl_inc <- read_csv("data/human_VL_incidence.csv")

# canine leish prevalence data
canine <- read_csv("data/canine_leish_prevalence.csv")

# bayesian models effect sizes
human_effects <- read_csv("output/05_human_model_effects.csv")
canine_effects <- read_csv("output/06_canine_model_effects.csv")


# Run ---------------------------------------------------------------------

# combine main results into single df
results <- suit %>% 
  mutate(CNTR_CODE = str_sub(NUTS_ID, end=2)) %>% 
  left_join(ctry, by = "CNTR_CODE")

#### 1. Maps with predicted suitability ####

# Figure 1: change in leish climatic suitability
fig1_data <- study_area %>% 
  left_join(results, by=c("NUTS_ID", "CNTR_CODE"))

fig1_data_endemic <- fig1_data %>% 
  filter(endemic_leish == 1) %>% 
  group_by(CNTR_CODE, endemic_leish) %>% 
  summarise(.groups="keep")

fig1 <- ggplot() +
  geom_sf(data=fig1_data, 
          aes(fill=factor(changebin_final, levels = c(0, -1, 2, 1))), lwd=0.1)+
  scale_fill_manual(values = c("#ffffd4","#fde0dd","#fa9fb5","#c51b8a"),
                    labels = c("Unsuitable in both periods",
                               "Suitable only in 2001-2010",
                               "Suitable in 2001-2010 and 2011-2020",
                               "Suitable only in 2011-2020"),
                    guide=guide_legend(reverse=TRUE),
                    name = "Leishmania infantum predicted suitability")+
  geom_sf(data=fig1_data_endemic, aes(col="darkblue"), alpha=0)+
  scale_color_identity(name = "Status of endemicity", 
                       labels = c(darkblue = "Endemic countries"), 
                       guide = "legend")+
  theme_bw(base_size=12)

# Figure 2: continuous climatic suitability in both periods
fig2_data <- study_area %>% 
  left_join(results, by="NUTS_ID") %>%
  select(NUTS_ID, leish_pred10, leish_pred20) %>% 
  rename("2001-2010" = "leish_pred10", 
         "2011-2020" = "leish_pred20") %>% 
  pivot_longer(cols= c("2001-2010", "2011-2020"),
               names_to = "period",
               values_to = "suitability")

fig2 <- ggplot(fig2_data) +
  geom_sf(aes(fill=suitability), lwd=0.1)+
  facet_wrap(~period, nrow=2)+
  scale_fill_gradient(low="#f1eef6", high = "#980043",
                      breaks = c(seq(0, 1, 0.1)),
                      name="Leishmania infantum \npredicted climatic suitability")+
  theme_bw(base_size=12)


#### 2. Suitable NUTS per country ####

# Table 1: number of suitable nuts by country
table1 <- results %>% 
  group_by(country_eng) %>% 
  drop_na() %>% 
  summarise(total_nuts3_regions = n(),
            suitable_nuts3_2010 = sum(leish_pred10_bin),
            suitable_nuts3_2020 = sum(leish_pred20_bin),
            .groups = "keep") %>% 
  mutate(percent_suitable_nuts3_2010 = 
           round((suitable_nuts3_2010/total_nuts3_regions)*100, 
                 digits=1),
         percent_suitable_nuts3_2020 = 
           round((suitable_nuts3_2020/total_nuts3_regions)*100, 
                 digits=1),
         percent_change = 
           round(percent_suitable_nuts3_2020-percent_suitable_nuts3_2010, 
                 digits=1)) %>% 
  filter(sum(suitable_nuts3_2010, suitable_nuts3_2020) > 0)

# Figure 3: number of suitable nuts by country (plot)
fig3_data <- results %>% 
  group_by(country_eng) %>% 
  summarise("2001-2010" = sum(leish_pred10_bin),
            "2011-2020" = sum(leish_pred20_bin),
            nuts_by_country = n()) %>% 
  pivot_longer(cols= c("2001-2010", "2011-2020"),
               names_to = "period",
               values_to = "nuts_with_leish") %>% 
  mutate(percent_nuts_with_leish = (nuts_with_leish/nuts_by_country)*100) %>%
  filter(nuts_with_leish >0)

# sorted list of countries by number of suitable nuts
cntr_sort <- fig3_data %>% 
  arrange(nuts_with_leish) %>% 
  select(country_eng) %>% 
  distinct() %>% 
  pull()

fig3 <- fig3_data %>% 
  mutate(countries= factor(country_eng,levels = c(cntr_sort))) %>% 
  mutate(Period = factor(period, levels = c("2011-2020", "2001-2010"))) %>% 
  ggplot(aes(fill=Period, y=nuts_with_leish, x=countries))+
  geom_bar(position= "dodge", stat = "identity", width=.7)+
  scale_fill_manual(values = c("#c51b8a","#fa9fb5"),
                    labels = c("2011-2020", "2001-2010"))+
  coord_flip()+
  theme_classic() +
  xlab("") +
  ylab("Number of suitable NUTS3 regions") +
  theme(panel.grid.major.x = element_line(colour = "grey", linewidth = 0.05), 
        axis.text.y=element_text(size=10),
        axis.text.x = element_text(size = 10), 
        axis.title.y=element_text(size=12),
        legend.title=element_text(size=12, face="bold"), 
        legend.text=element_text(size=10), 
        plot.title=element_text(size=14, face="bold")) +
  guides(fill = guide_legend(reverse = TRUE))


#### 3. Aggregates by socioeconomic vulnerability ####

# join AROPE rates at NUTS2 level to leish predictions
fig4_data <- suit %>% 
  mutate(NUTS2_ID = str_sub(NUTS_ID, start = 1, end = -2)) %>% 
  # join arope rates
  left_join(arope, by =c("NUTS2_ID" = "NUTS_ID")) %>%
  mutate(AROPE = factor(classif, levels = c("low","medium","high"))) %>%
  drop_na(AROPE)

# Figure 4: climatic suitability by vulnerability levels
fig4 <- fig4_data %>% 
  ggplot(aes(x=AROPE, y=leish_pred20, fill=AROPE)) +
  geom_boxplot(width=.3, fill="white") +
  theme_classic() +
  xlab("Socioeconomic vulnerability") +
  ylab("Average climatic suitability") +
  theme(axis.text.y=element_text(size=10),
        axis.text.x = element_text(size = 10), 
        axis.title.y=element_text(size=12),
        axis.title.x = element_text(size = 12), 
        legend.position = "none")


#### 4. Associations with human and canine leishmaniasis ####

# rebuild df used in human disease model
dt.suit <- suit %>% 
  select(NUTS_ID, leish_pred20, leish_pred10) %>% 
  mutate(across(2:3, ~.x*10)) %>% 
  rowwise() %>% 
  mutate(leish_avg=mean(c(leish_pred20, leish_pred10))) %>% 
  ungroup()

human_df <- vl_inc %>% 
  left_join(dt.suit, by="NUTS_ID") %>% 
  # Assign suitability to different periods and countries
  mutate(leish=case_when(country == "France" ~ leish_pred10,
                         country == "Greece" ~ leish_pred20,
                         country == "Spain" ~ leish_avg,
                         country == "Italy" ~ leish_pred20,
                         TRUE ~ NA)) %>% 
  select(NUTS_ID, country, cases, population, leish)

canine_df <- canine %>% 
  left_join(dt.suit, by="NUTS_ID") %>% 
  # Assign suitability to different periods and countries
  mutate(leish=case_when(country == "Spain" ~ leish_pred20,
                         country == "Portugal" & period == 2009 ~ leish_pred10,
                         country == "Portugal" & period == 2021 ~ leish_pred20)) %>% 
  select(NUTS_ID, country, tested_dogs, positive_dogs, prevalence, leish)

# country names for human and canine models
nms_human <- c("FR", "EL", "IT", "ES")
nms_canine <- c("ES", "PT")

# Plot human model effect sizes
pt.effects.human <- human_effects %>% 
  filter(!grepl("Intercept", coefficient)) %>% 
  mutate(country=case_when(grepl("FR", model) ~ "France",
                           grepl("ES", model) ~ "Spain",
                           grepl("EL", model) ~ "Greece", 
                           grepl("IT", model) ~ "Italy", TRUE ~ NA)) %>% 
  ggplot() +
  geom_pointrange(aes(x=country, y=mean, ymin=lower, ymax=upper),
                  shape=18, size=1) +
  geom_hline(yintercept=1, linetype="dashed", alpha=0.5) +
  labs(x="", y="Effect size") +
  coord_flip() +
  theme_classic() +
  theme(axis.text.y=element_text(size=10),
        axis.text.x = element_text(size = 10), 
        axis.title.y=element_text(size=12),
        axis.title.x = element_text(size = 12))

# Plot human disease vs suitability
pt.scatter.human <- human_df %>% 
  mutate(inc=(cases/population)*100000) %>%
  drop_na() %>% 
  ggplot(aes(x=leish/10)) +
  geom_point(aes(y=inc)) +
  facet_wrap(~country, scales="free") +
  labs(x="Climatic suitability", y="HumL incidence per NUTS3 region") +
  theme_classic()+
  theme(axis.text.y=element_text(size=10),
        axis.text.x = element_text(size = 10), 
        axis.title.y=element_text(size=12),
        axis.title.x = element_text(size = 12))

# Plot canine model effect sizes
pt.effects.canine <- canine_effects %>% 
  filter(!grepl("Intercept", coefficient)) %>% 
  mutate(country=case_when(grepl("ES", model) ~ "Spain",
                           grepl("PT", model) ~ "Portugal")) %>% 
  ggplot() +
  geom_pointrange(aes(x=country, y=mean, ymin=lower, ymax=upper),
                  shape=18, size=1) +
  geom_hline(yintercept=1, linetype="dashed", alpha=0.5) +
  labs(x="", y="Effect size") +
  coord_flip() +
  theme_classic() +
  theme(axis.text.y=element_text(size=10),
        axis.text.x = element_text(size = 10), 
        axis.title.y=element_text(size=12),
        axis.title.x = element_text(size = 12))

# Plot canine prevalence vs suitability
pt.scatter.canine <- canine_df %>% 
  mutate(prev=(positive_dogs/tested_dogs)*100) %>%
  ggplot(aes(x=leish/10)) +
  geom_point(aes(y=prev)) +
  facet_wrap(~country, scales="free") +
  labs(x="Climatic suitability", y="CanL prevalence per NUTS3 region") +
  theme_classic()+
  theme(axis.text.y=element_text(size=10),
        axis.text.x = element_text(size = 10), 
        axis.title.y=element_text(size=12),
        axis.title.x = element_text(size = 12))

# Final plot
right.plot1 <- cowplot::plot_grid(pt.effects.human, nrow=2)
right.plot2 <- cowplot::plot_grid(pt.effects.canine, nrow=2)

fig5 <- plot_grid(pt.scatter.human, right.plot1, 
                  pt.scatter.canine, right.plot2,
                  rel_widths=c(1,0.5),
                  rel_heights=c(1.5,1),
                  labels=c("a)", "b)", "c)", "d)"))


# Data output -------------------------------------------------------------

# Figure 1
ggsave(fig1, filename = "figs/07_main_fig1.png",
       width=10, height=5, units = "in")

# Figure 2
ggsave(fig2, filename = "figs/07_main_fig2.png",
       width=9, height=10, units = "in")

# Table 1
write_csv(table1, "output/07_main_table1.csv")

# Figure 3
ggsave(fig3, filename = "figs/07_main_fig3.png",
       width=6, height=5, units = "in")

# Figure 4 (png)
ggsave(fig4, filename = "figs/07_main_fig4.png",
       width=5, height=5, units = "in")

# Figure 4 (pdf)
pdf(file = "figs/07_main_fig4.pdf", width=5, height=5)
fig4
dev.off()

# Figure 5 (png)
ggsave(fig5, filename = "figs/07_main_fig5.png",
       width=10, height=10, units = "in")

# Figure 5 (pdf)
pdf(file = "figs/07_main_fig5.pdf",
    width=10, height=10)
fig5
dev.off()

#END
